#ifndef _CLUT_FXH_
#define _CLUT_FXH_
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//  CLut effect11 by kingeric1992
//                                  Nov.4.2017
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "enbInternals.fxh"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  internals
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define CLUT_FORMAT_NULL        0
#define CLUT_FORMAT_16BIT       1
#define CLUT_FORMAT_32BIT       2
#define CLUT_FORMAT_64BIT       3
#define CLUT_FORMAT_64BIT_ALT   4
#define CLUT_FORMAT_256BIT      5

static const uint3 CLut_Format[6] = {
    {    1,   1, 0 },
    {  256,  16, 4 },
    { 1024,  32, 5 },
    { 4096,  64, 6 },
    {  512, 512, 6 },
    { 4096,4096, 8 }
};

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  export function
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

struct CLutStruct {
    bool      State;
    float     Weight;
    uint      format;
    Texture2D Tex;
};

float3 CLut( float3 col, CLutStruct IN)
{
    if(!IN.State) return col;
    
    float3 format     = CLut_Format[IN.format];
    float2 CLut_pSize = 1.0 / format.xy;
    float4 CLut_UV    = 0.0;
           
           col        = saturate(col) * ((1 << (int)format.z)  - 1.0);
           col.rg     = (col.rg + 0.5) * CLut_pSize;
           
    float2 CLut_bID    = floor(col.b);
           CLut_bID.y += 1.0;

    if(IN.format < 4) {
        CLut_UV.xz = col.xx + CLut_bID.xy * CLut_pSize.y;
        CLut_UV.yw = col.yy;
    }
    else {
        int grid = (int)format.x / (1 << (int)format.z);
    
        CLut_UV.xz = col.xx + floor(CLut_bID % grid) / float(grid);
        CLut_UV.yw = col.yy + floor(CLut_bID / grid) / float(grid);
    }    
    return lerp( IN.Tex.SampleLevel(SamplerLinear, CLut_UV.xy, 0).rgb, 
                 IN.Tex.SampleLevel(SamplerLinear, CLut_UV.zw, 0).rgb, col.b - CLut_bID.x);
}

#endif //_CLUT_FXH_